var send_comment_replied_to_parent = (parent_comment_id, child_comment) =>{
	var cmt = get_comment(parent_comment_id);
	if (is_nv_error(cmt)) {return;}
	var user = cmt.user_id > 0
				? get_user_by_id(cmt.user_id)
				: {
					name: cmt.name,
					email: cmt.email
				};
	if (!user.email) {return;}
	var nv_home_url = get_option('nv_home_url','');
	var nv_site_name = get_option('nv_site_name','nvPress');
	var {title,post_type,slug} = get_post(cmt.post_id);
	var mail_title = `您在《${title || '未命名'}》的留言有了回复`;
	var mail_paragraphs = [
		`您曾在《${title || '未命名'}》留言：`,
		`<span class="quote">
			${cmt.content.replace(/\n\n+/g,'\n').split('\n').map(p=>`<span style="margin: 15px 0;display:block">${esc_html(p)}</span>`).join('')}
		</span>`,
		`${child_comment.name} 给您的回复：`,
		`<span class="quote">
			${child_comment.content.replace(/\n\n+/g,'\n').split('\n').map(p=>`<span style="margin: 15px 0;display:block">${esc_html(p)}</span>`).join('')}
		</span>`,
		`欢迎再次光临<a href="${ nv_home_url }" target="_blank" class="blue">${ nv_site_name }</a>。`,
		`(此邮件由系统自动发出, 请勿回复)`
	];
	send_mail_to_user(user, mail_title, mail_paragraphs)
	.then(()=>{})
	.catch(err=>{console.log('评论被回复，发送邮件通知失败：',err)})
}

export default ()=>{
	add_action('nv_set_comments_status',function(before_comments,target_status) {
		if (target_status == 'publish' && get_option('comment_published_mail_to_author')) {
			var post_temp = {};
			var comment_previously_approved = get_option('comment_previously_approved') ? '，再次使用该邮箱评论将自动通过审核。' : '。';
			var nv_home_url = get_option('nv_home_url','');
			var nv_site_name = get_option('nv_site_name','nvPress');
			var comment_replied_mail_to_author = get_option('comment_replied_mail_to_author');

			before_comments.forEach(cmt=>{
				var user = cmt.user_id > 0
							? get_user_by_id(cmt.user_id)
							: {
								name: cmt.name,
								email: cmt.email
							};
				if (!user.email) {return;}
				if (!post_temp[cmt.post_id]) {
					var {title,post_type,slug} = get_post(cmt.post_id);
					post_temp[cmt.post_id] = {title,post_type,slug}
				}
				var mail_title = `您在《${post_temp[cmt.post_id].title || '未命名'}》的留言已通过审核`;
				var mail_paragraphs = [
					`您曾在《${post_temp[cmt.post_id].title || '未命名'}》留言：`,
					`<span class="quote">
						${cmt.content.replace(/\n\n+/g,'\n').split('\n').map(p=>`<span style="margin: 15px 0;display:block">${esc_html(p)}</span>`).join('')}
					</span>`,
					`留言内容已通过系统审核${ comment_previously_approved }`,
					`欢迎再次光临<a href="${ nv_home_url }" target="_blank" class="blue">${ nv_site_name }</a>。`,
					`(此邮件由系统自动发出, 请勿回复)`
				];
				send_mail_to_user(user, mail_title, mail_paragraphs)
				.then(()=>{})
				.catch(err=>{console.log('评论审核通过，发送邮件通知失败：',err)})

				if (cmt.parent > 0 && comment_replied_mail_to_author) {
					// 发送评论已回复邮件
					send_comment_replied_to_parent(cmt.parent, cmt);
				}
			})
		}
	});

	add_action('nv_comment_inserted', function(cmt) {
		if (cmt.parent > 0 && cmt.status == 'publish' && get_option('comment_replied_mail_to_author')) {
			// 发送评论已回复邮件
			send_comment_replied_to_parent(cmt.parent, cmt);
		}

		// 只要不是垃圾评论，[正常评论、待审评论]，都通知文章作者。(作者自己评论就不通知了)
		if ( ['publish','pending'].includes(cmt.status) ) {
			// 查询文章作者
			var post = get_post(cmt.post_id);
			if ( is_nv_error(post) ) { return; }

			var author_id = post.author;
			if (author_id == cmt.user_id) { return; }

			var author = get_user_by_id(author_id);
			if (!author.email) { return; }

			// 评论者信息
			var user = cmt.user_id > 0
					? get_user_by_id(cmt.user_id)
					: {
						name: cmt.name,
						email: cmt.email,
						url: cmt.url,
					};

			var title = `${cmt.status == 'publish' ? '评论' : '请审核'}：“${post.title || '未标题'}”`;
			var url = encodeURI(`${get_option('nv_home_url','')}/nv-admin/comments?s={${cmt.status=='pending' ? `"status":"pending",` : ''}"post":${post.id},"title":"《${post.title}》的评论："}`);

			var reply = [];
			if (cmt.parent) {
				var parent_cmt = get_comment(cmt.parent);
				if (!is_nv_error(parent_cmt)) {
					var parent_user = parent_cmt.user_id > 0
						? get_user_by_id(parent_cmt.user_id)
						: {
							name: parent_cmt.name,
						};
					reply.push(`这条评论回复给 <b>${parent_user.name}</b>，原评论内容如下：`)
					reply.push(`<span class="quote">
									${parent_cmt.content.replace(/\n\n+/g,'\n').split('\n').map(p=>`<span style="margin: 15px 0;display:block">${esc_html(p)}</span>`).join('')}
								</span>`)
				}
			}

			send_mail_to_admin(title, [
				`您的文章《${post.title || '未标题'}》有了新评论`,
				`评论者：<b>${user.name}</b> ${user.email ? `<${user.email}>`: ''}`,
				...(user.url ? [`URL：${user.url}`] : []),
				`内容：`,
				`<span class="quote">
					${cmt.content.replace(/\n\n+/g,'\n').split('\n').map(p=>`<span style="margin: 15px 0;display:block">${esc_html(p)}</span>`).join('')}
				</span>`,
				...reply,
				`${cmt.status=='pending' ? `去审核` : '查看该文章所有评论'}：<a class="blue" target="_blank" href="${url}">${url}</a>`
			])
			.then(()=>{})
			.catch(err=>{console.log('新增评论，发送邮件通知文章作者失败：',err)})
		}

	});
}