// 系统自带的块渲染。
var nv_create_class_attribute = opt => {
	var classNames = [];
	Object.keys(opt).forEach(key=>{
		if (opt[key]) {
			classNames.push(key)
		}
	})
	return classNames.length
			? ` class="${classNames.join(' ')}"`
			: '';
}
var nv_create_style_attribute = opt => {
	var styles = [];
	Object.keys(opt).forEach(key=>{
		var value = opt[key];
		if ( ![false,undefined,null,''].includes(value) ) {
			// 把style改成短横线命名方式
			var kebab_case_key = key
								.replace(/([a-z])([A-Z])/g, "$1-$2")
								.replace(/[\s_]+/g, '-')
								.toLowerCase();
			styles.push(`${kebab_case_key}:${value}`)
		}
	})
	return styles.length
			? ` style="${styles.join(';')}"`
			: '';
}

export default ()=>{
	add_filter(`async:render_block:paragraph`,(next, last_filtered, {data})=>{
		if ( apply_filters('render_core_block:paragraph',true) ) {
			var classAttr = nv_create_class_attribute({'drop-cap': data.dropCap});
			var styleAttr = nv_create_style_attribute({
				fontSize: `${data.fontSize}px`,
				color: data.color,
				textAlign: data.align == 'left' ? '' : data.align,
				fontWeight: `${data.fontWeight}`,
				textIndent: data.textIndent
			});
			next(`<p${classAttr}${styleAttr}>${data.text}</p>`);
		} else {
			next(``);
		}
	})

	add_filter(`async:render_block:header`,(next, last_filtered, {data})=>{
		if ( apply_filters('render_core_block:header',true) ) {
			var styleAttr = nv_create_style_attribute({
				fontSize: `${data.fontSize}px`,
				color: data.color,
				fontWeight: `${data.fontWeight}`
			})
			next(`<h${data.level}${styleAttr}>${data.text}</h${data.level}>`);
		} else {
			next(``);
		}
	})

	add_filter(`async:render_block:image`,(next, last_filtered, {data})=>{
		if ( apply_filters('render_core_block:image',true) ) {
			var figureClassAttr = nv_create_class_attribute({
				'image-align-left':data.align=='left',
				'image-align-center':data.align=='center',
				'image-align-right':data.align=='right',
				'image-align-full':data.align=='full',
			});
			var classAttr = nv_create_class_attribute({
				rounded: data.rounded,
				shadow: data.shadow,
			});
			var styleAttr = nv_create_style_attribute({
				width: (data.align !== 'full') ? data.width ? data.width+'px' : 'auto' : false,
				height: (data.align !== 'full') ? data.height ? data.height+'px' : 'auto' : false,
			})
			next(
				data.href
				? `<figure${figureClassAttr}><a href="${data.href}" target="${data.opennew ? "_blank" : "_self"}">
						<img${classAttr}${styleAttr} src="${data.src}" />
					</a></figure>
					`
				: `<figure${figureClassAttr}><img${classAttr}${styleAttr} src="${data.src}" /></figure>`
			);
		} else {
			next(``);
		}
	})

	add_filter(`async:render_block:list`,(next, last_filtered, {data})=>{
		if ( apply_filters('render_core_block:list',true) ) {
			var styleAttr = nv_create_style_attribute({
				fontSize: `${data.fontSize}px`,
				color: data.color,
				fontWeight: `${data.fontWeight}`,
			})
			var startAttr = ` start="${data.start || 0}"`;
			var get_list = (data, depth) => {
				var result = "";
				var tag = data.style == 'ordered' ? 'ol' : 'ul';
				var list = data.items.map(item=>{
					return `<li>
					${item.content}
					${ (item.items && item.items.length > 0) ? get_list({style: data.style, items: item.items}) : '' }
					</li>`
				}).join("");
				if (depth == 0) {
					return `<${tag}${styleAttr}${startAttr}>${list}</${tag}>`;
				} else {
					return `<${tag}>${list}</${tag}>`;
				}
				
			}
			next( get_list(data, 0) );
		} else {
			next(``);
		}
	})

	add_filter(`async:render_block:table`,(next, last_filtered, {data})=>{
		if ( apply_filters('render_core_block:table',true) ) {
			var classAttr = nv_create_class_attribute({
				'width-footings': data.withFootings,
				'layout-fixed': data.layoutFixed,
			});

			var result = `<table width="100%"${classAttr}>`;
			if (data.withHeadings && data.content[0]) {
				result += '<thead><tr>'
				data.content[0].forEach(col=>{result += `<th>${col}</th>`})
				result += '</tr></thead>';
			}
			result += "<tbody>";
			var content = data.withHeadings ? data.content.slice(1) : data.content;
			content.forEach(tr=>{
				result += "<tr>"
				tr.forEach(col=>{result += `<td>${col}</td>`})
				result += '</tr>'
			})
			result += '</tbody></table>'
			next(result)
		} else {
			next(``);
		}
	})

	add_filter(`async:render_block:quote`,(next, last_filtered, {data})=>{
		if ( apply_filters('render_core_block:quote',true) ) {
			next(`<figure rel="quote">
				<blockquote>${data.text}</blockquote>
				<figcaption>${data.caption}</figcaption>
				</figure>`)
		} else {
			next(``);
		}
	})

	add_filter(`async:render_block:delimiter`,(next, last_filtered, {data})=>{
		if ( apply_filters('render_core_block:delimiter',true) ) {
			var classAttr = nv_create_class_attribute({
				'hide-symbol': data.hideSymbol,
			});
			var styleAttr = nv_create_style_attribute({
				height: data.customHeight ? data.customHeight+'px' : false,
			})
			next(`<hr${classAttr}${styleAttr} />`)
		} else {
			next(``);
		}
	})

	add_filter(`async:render_block:code`,(next, last_filtered, {data})=>{
		if ( apply_filters('render_core_block:code',true) ) {
			var languageAttr = data.language ? ` language="${data.language}"` : '';
			next(`<pre${languageAttr}><code>${esc_html(data.pre)}</code></pre>`)
		} else {
			next(``);
		}
	})

	add_filter(`async:render_block:raw`,(next, last_filtered, {data})=>{
		if ( apply_filters('render_core_block:raw',true) ) {
			next(`${data.html}`)
		} else {
			next(``);
		}
	})
}