import * as path from "path"
import {getDateTime} from "./method-media.js"
import sharp from "sharp"

export default ()=>{
	global._sharp_installed = true;

	var generate_image_thumbnail = (imagePath,thumbnailPath,maxSize,callback)=> {
		return new Promise((resolve,reject)=>{
			var image = sharp(imagePath);
			image.metadata()
			.then( ({width,height}) =>{
				var max = Math.max(width,height);
				if (max > maxSize) {
					var ratio = maxSize / max;
					image
					.resize( parseInt(width * ratio),parseInt(height * ratio) )
					.webp({quality:83,smartSubsample:true})
					.toFile(thumbnailPath, (err, info) => {
						if (err) { reject(err) }
						else {
							if (callback) {callback(info);}
							resolve(info);
						}
					});
				} else {
					reject();
				}
			})
			.catch(err=>{
				reject(err);
			})
		})
	}

	var get_image_size = (imagePath,callback)=>{
		return new Promise((resolve,reject)=>{
			sharp(imagePath)
			.metadata()
			.then( info =>{
				if (callback) {callback(info.width, info.height);}
				resolve(info);
			})
			.catch(err=>{
				reject(err);
			})
		})
	}

	var add_deal_list = (func,args) => {
		if ( !(func._list instanceof Array) ) { func._list = [] }
		func._list.push(args)
		var dealFunc = ()=>{
			var currentArgs = func._list.shift();
			func( ...currentArgs )
			.catch(err=>{
				console.log(err)
			})
			.finally(()=>{
				if (func._list.length > 0) {
					dealFunc();
				} else {
					func._dealing = false;
				}
			})
		}
		if (!func._dealing) {
			func._dealing = true;
			dealFunc();
		}
	}

	// Sharp 仅这些扩展名才当做缩略图处理：https://sharp.pixelplumbing.com/api-output
	var image_extensions = ['.jpg','.jpeg','.png','.webp','.gif','.jp2','.tiff','.avif','.heif','.raw','.tile'];

	//读取图片尺寸
	add_action("media_insert",(media_id, {created_time,filename})=>{
		if ( !sharp || !image_extensions.includes( path.extname(filename) ) ) {return;}
		
		var {year,month} = getDateTime(created_time);
		var imagePath = path.join(process.cwd(),`./nv-content/uploads/${year}/${month}/${filename}`);
		add_deal_list( get_image_size, [imagePath,(width,height)=>{
			update_media_meta(media_id,'width',width);
			update_media_meta(media_id,'height',height);
		}]);
	});

	//生成缩略图
	add_action("media_insert",(media_id, {created_time,filename})=>{
		if ( !sharp || !image_extensions.includes( path.extname(filename) ) ) {return;}

		var auto_generate_thumbnail = get_option("auto_generate_thumbnail");
		var auto_generate_middlenail = get_option("auto_generate_middlenail");

		if (auto_generate_thumbnail || auto_generate_middlenail) {
			var {year,month} = getDateTime(created_time);
			var imagePath = path.join(process.cwd(),`./nv-content/uploads/${year}/${month}/${filename}`);
		}
		if (auto_generate_thumbnail) {
			var thumbnailPath = `${imagePath}.thumbnail.webp`;
			add_deal_list( generate_image_thumbnail, [imagePath,thumbnailPath,320,info=>{
				//webp是1，avif是2。如果以后浏览器都支持更小体积的avif后，则修改编码为avif
				update_media_meta(media_id,'hasThumbnail',1);
			}]);
		}
		if (auto_generate_middlenail) {
			var middlenailPath = `${imagePath}.middle.webp`;
			add_deal_list( generate_image_thumbnail, [imagePath,middlenailPath,1024,info=>{
				//webp是1，avif是2。如果以后浏览器都支持更小体积的avif后，则修改编码为avif
				update_media_meta(media_id,'hasMiddle',1);
			}]);
		}
	});
}