export default ()=>{
	add_action('async:_nv_init',next=>{
		var nv_theme_modules = get_registered_theme_modules();
		if ( Object.keys(nv_theme_modules).length ) {
			// 注册文章类型：用 theme_module 来作为边栏的文章类型。不显示UI！
			register_posttype('theme_module',{
				label: "主题模块",
				labels: {
					singular_name: "主题模块",
					all_items: "所有主题模块",
					add_new_item: "添加主题模块",
				},
				show_ui: false,
				supports: ['title','editor'],
				hierarchical: false,
			})
			// 在外观下在增加边栏菜单，使用前端内置的postlist组件（slug的-后面是post_type：theme_module）
			add_submenu_page({
				parent_slug: "appearance",
				page_title: "主题模块",
				menu_title: "主题模块",
				menu_slug: "theme-theme_module",
				power: 10,
				position: 9,
				_component: "postlist",
			})
			// 由于不显示UI，默认不会注册编辑器，因此手动增加一个theme_module编辑器，使用前端内置的 editor 组件（slug的-后面是post_type：theme_module）
			add_filter('_nv_admin_routes',(routes, currentUserPower)=>{
				routes = routes || [];
				if (currentUserPower >= 10) {
					routes.push({
						page_title: "编辑主题模块",
						menu_slug: "editor-theme_module",
						_component: "editor",
						highlight_menu: "theme-theme_module"
					});
				}
				return routes;
			});
			// 在文章列表页新增显示边栏归属位置的“列”（边栏一般有多个，所以需要显示出来具体是用在何处）
			// 这个列被命名为：_theme_module_position，接下来会在读取文章数据的时候将 _theme_module_position 的数据加上去才能显示
			add_filter('manage_theme_module_posts_columns',columns=>{
				return [
					...columns,
					{key: '_theme_module_position', title: '显示位置'}
				]
			})

			// 添加 _theme_module_position 需要显示的具体内容
			add_filter('manage_posts_columns',(post,post_type)=>{

				// 只处理theme_module的文章内容，不是theme_module的文章类型不需要添加这一列的数据
				if (post_type !== 'theme_module') { return {}; }

				// 从保存的meta中读取数据
				var position_val = get_post_meta(post.id,'_theme_module_position');
				var theme_module_name = nv_theme_modules[position_val];
				var _theme_module_position = "未使用";
				if (theme_module_name) {
					_theme_module_position = post.status == 'publish' ? theme_module_name : `发布后显示在：${theme_module_name}`
				}
				return {_theme_module_position}
			})

			// 编辑器页面增加 “显示位置” 的选项
			add_meta_box({
				post_types: ['theme_module'],
				meta_box_slug: "theme_module-metabox-position",
				content: {
					title: '显示位置',
					items: [{
						label: "", type: "Select", meta_key: "_theme_module_position", config: {
							options: Object.keys(nv_theme_modules).map(key=>({ label: nv_theme_modules[key], value: key })),
							'fallback-option': false,
							clearable: true,
							placeholder: "未使用"
						},
					}]
				},
			})

			// 多个文章同时显示在同一个位置的处理：只让最后一个保存的显示在那儿，其它的都改为未显示（删除 _theme_module_position 元数据）
			// 把过滤器当钩子使用。数据不变更，但是修改别的数据（此处应该强制修改，否则 update_metadata 会重复调用，导致死循环）
			// 注意：这个钩子是在元数据update前过滤数据的哦
			add_filter(`update_post_metadata`, (value, object_id, key)=>{
				if (key !== "_theme_module_position") { return value }

				var post = get_post(object_id);
				// 前提是，这个object_id是theme_module，并且已发布
				if (post.post_type == 'theme_module' && post.status == 'publish') {
					// 如果key是 _theme_module_position，那么将所有和 value 相同的数据 value 设为空
					nvdb.postmeta.setColumn(r=>r.key == "_theme_module_position" && r.value == value,{value: null})
				}

				// 由于不篡改数据，因此直接return原数据
				return value;
			});
		}
		next();
	},11);
}