import * as fs from 'fs'
import * as path from 'path'
import minimist from "minimist";
import {createHttpTerminator} from 'http-terminator';
import init_core_update_hooks from './init_core_update.js'
import init_themes from "./init-theme.js"
import init_plugins from "./init-plugins.js"
import compression from "compression"
import cookieParser from "cookie-parser"
import express from "express"
import expressWs from "express-ws"
import {_get_static_urls_options,_get_rest_routes_options,_get_socket_routes_options} from "./method-common.js";

// apis
import init_default_menu from "./init_default_menu.js";
import init_default_api from "./init_default_api.js";
import init_default_routes_api from "./init_default_routes_api.js";
import init_default_users_api from "./init_default_users_api.js";
import init_default_user_invite_api from "./init_default_user_invite_api.js";
import init_default_options_api from "./init_default_options_api.js";
import init_default_mail_api from "./init_default_mail_api.js";
import init_default_media_api from "./init_default_media_api.js";
import init_default_taxonomies_api from "./init_default_taxonomies_api.js";
import init_default_post_api from "./init_default_post_api.js";
import init_default_editor_api from "./init_default_editor_api.js";
import init_default_comments_api from "./init_default_comments_api.js";
import init_default_theme_api from "./init_default_theme_api.js";
import init_default_plugin_api from "./init_default_plugin_api.js";
import init_default_nav_menu_api from "./init_default_nav_menu_api.js";
import init_default_dashboard_api from "./init_default_dashboard_api.js";
import init_default_theme_module_api from "./init_default_theme_module_api.js";
import init_default_package_api from "./init_default_package_api.js";
import init_default_realtime_api from "./init_default_realtime_api.js";
import init_frontstage_api from "./init_frontstage_api.js";

// hooks
import hooks_media from "./hooks-media.js";
import hooks_user from "./hooks-user.js";
import hooks_comments from "./hooks-comments.js";
import hooks_theme_module_hooks from "./hooks-theme-module.js";


export default ()=>{
	const args = minimist(process.argv)
	init_core_update_hooks();

	add_action('async:_nv_init',next=>{
		// 每次启动都清理temp
		nv_remove_folder('./temp');
		next();
	})

	//钩子创建原生的内容
	add_action('async:_nv_init',async next=>{
		add_action('init_express',()=>{
			//前台（从主题里面去注册）
			// register_static_url('/', path.join(process.cwd(),"./nv-frontpage/dist/") );

			//后台
			// ！！！！！！！！！！！！！！！！！！！！！！！！！
			// 如果要修改后台地址，那么，后台编译时也要修改 base
			// ！！！！！！！！！！！！！！！！！！！！！！！！！
			register_static_url('/nv-admin', path.join(process.cwd(),"./nv-admin/"));
			//上传文件夹
			register_static_url('/uploads', path.join(process.cwd(),"./nv-content/uploads/") );
		});
		add_filter('rewrite_rules_array',arr=>{
			arr.push({
				from: /\/nv-admin/, to: '/nv-admin/index.html'
			})
			return arr;
		});

		//创建菜单
		init_default_menu();
		//创建api
		init_default_api();
		//创建router-api
		init_default_routes_api();
		//创建用户-api
		init_default_users_api();
		//邀请码-api
		init_default_user_invite_api();
		//创建Options-api
		init_default_options_api();
		//创建Mail-api
		init_default_mail_api();
		//创建媒体-api
		init_default_media_api()
		//创建分类法-api
		init_default_taxonomies_api();
		//创建文章-api
		init_default_post_api();
		// 编辑器API
		init_default_editor_api();
		//创建评论-api
		init_default_comments_api();
		//创建主题-api
		init_default_theme_api();
		//创建插件-api
		init_default_plugin_api();
		//创建导航菜单-api
		init_default_nav_menu_api();
		//创建仪表盘api
		init_default_dashboard_api();
		//创建边栏api
		init_default_theme_module_api();
		//创建套件中心api
		init_default_package_api();
		//创建Socket实时信息-API
		init_default_realtime_api();
		//创建前台-API
		init_frontstage_api();

		//默认钩子
		hooks_media();
		hooks_user();
		hooks_comments();
		hooks_theme_module_hooks();

		next();
	},0);


	// plugins
	add_action('async:_nv_init',next=>{
		nv_create_dirs("./nv-plugins")
		init_plugins()
		.finally(()=>{
			next();
		})
	},5);

	// theme
	add_action('async:_nv_init',next=>{
		nv_create_dirs("./nv-themes")

		init_themes()
		.finally(()=>{
			next();
		})

	});

	//exoress
	add_action('async:_nv_init',next=>{
		// 曾经的_nv_init不再使用。为了防止老的主题插件异常，在这里加上
		do_action('_nv_init');

		do_action('init_express');
		var app = express();

		expressWs(app); //将socket服务混入app，为app添加.ws方法

		app.use(compression());
		app.use(cookieParser());

		do_action('register_client_side_render_router', app);
		
		// 解析 application/json
		app.use(express.json({limit: apply_filters('nv_express_limit_size','5mb') }));
		//处理Static静态资源
		const static_urls = _get_static_urls_options();
		static_urls.forEach(opt=>{
			var {url,dir} = opt;
			app.use(url,express.static( dir ))
		})

		function send_json(res, data, req, methods, namespace, route) {
			data = apply_filters(`rest_send_${methods}:/${namespace}${route ? `/${route}`: ''}`, data, req);
			res.json( is_nv_error(data) ? {
				error: data.message,
				error_code: data.code,
			} : data);
		}

		//定义rest api
		const rest_routes = _get_rest_routes_options();
		rest_routes.forEach(opt=>{
			var {namespace,route,methods,callback,permission_callback} = opt;
			app[methods](`/${namespace}${route ? `/${route}`: ''}`, (req,res)=>{
				// 预过滤处理req，包含header、body等信息
				req = apply_filters(`rest_request_${methods}:/${namespace}${route ? `/${route}`: ''}`, req);

				/*console.log(req.ip)
				console.log(req.headers['user-agent'])*/

				if (typeof(permission_callback) == 'function') {
					var result = get_user_by_nonce(req.cookies.nvnonce);
					var user = is_nv_error(result) ? {} : result;
					if ( !permission_callback(user.power || 0) ) {
						res.status(401);
						res.clearCookie("nvnonce",apply_filters('nv_user_logout_cookie_options',{
							path: "/",
							sameSite: "Lax",
						}))
						res.json({message: 'Permission declined!'});
						return;
					}
				}

				var data = methods == 'get' ? req.query : req.body;
				var result = callback(data, req, res);
				if ( result !== null
					&& typeof(result) == 'object'
					&& typeof(result.then) == 'function'
					&& typeof(result.catch) == 'function'
					&& typeof(result.finally) == 'function'
					) {
					// 是 promise， 采用异步回调。
					result
					.then(promise_data => {
						send_json(res, promise_data, req, methods, namespace, route)
					})
					.catch(promise_err => {
						send_json(
							res,
							new NV_Error(`${promise_err.name ? promise_err.name+": " : ''}${promise_err.message}`, promise_err.code),
							req,methods, namespace, route
							);
					})
				}
				else if (result !== undefined) {
					// 同步
					send_json(res, result, req, methods, namespace, route)
					return;
				}
			})
		})

		//定义socket api
		const socket_routes = _get_socket_routes_options();
		socket_routes.forEach(opt=>{
			var {namespace,route,methods,callback,permission_callback} = opt;
			app.ws(`/${namespace}${route ? `/${route}`: ''}`, (ws, req)=>{
				if (typeof(permission_callback) == 'function') {
					var result = get_user_by_nonce( req.headers['sec-websocket-protocol'] );
					var user = is_nv_error(result) ? {} : result;
					if ( !permission_callback(user.power || 0) ) {
						ws.close(4010,'Permission declined!')
						return;
					}
				}
				callback(ws, req);
			})
		})

		do_action('register_server_side_render_router', app);

		// 启动服务
		var nv_express_server = app.listen( args['port'] || 8081, function () {
			var port = nv_express_server.address().port
			nv_log(
				'后台地址：',
				`${nv_log.info('http://localhost')}${nv_log.bright(`:${port}`)}${nv_log.info('/nv-admin')}`,
				"\n启动成功：",
				nv_log.success(`${new Date().getTime() - global.nv_start_timestamp.getTime()}ms`)
			)
		})
		global._reboot_service = (message)=>{
			console.log('网络服务关闭中...');
			var httpTerminator = createHttpTerminator({
				server: nv_express_server,
			});
			httpTerminator.terminate()
			.then(()=>{
				// 9秒后通知主进程，重启Express。防止数据未正常存储导致数据丢失
				setTimeout(()=>{
					process.send('reboot_nvPress')
					process.exit();
				},9000)
			})
		}
		global._turn_off_service = (message)=>{
			console.log('网络服务关闭中...');
			var httpTerminator = createHttpTerminator({
				server: nv_express_server,
			});
			httpTerminator.terminate()
			.then(()=>{
				// 9秒后通知主进程，kill express
				setTimeout(()=>{
					process.send('turnoff_nvPress')
					process.exit();
				},9000)
			})
		}

		// 接收主进程消息
		process.on("message",(args)=>{
			// Docker关闭时，主进程通知子进程关闭
			if (args == 'Docker_Close_Container') {
				_turn_off_service();
			}
		})

		next();
	},20);
}

