import * as path from 'path'
import {v4 as uuidv4} from "uuid"

export default ()=>{
	nv_create_dirs('./nv-content/data/');

	const db_invites = new NV_Database(path.join(process.cwd(),"./nv-content/data/invites.json"),{
		columns: ['code','power','created_time','deprecate_time','use_time','user_id'],
		index: 'id'
	});
	nvdb.invites = db_invites;

	global.nv_create_invite = ( power, deprecate_time )=>{
		deprecate_time = parseInt(deprecate_time);
		if (!power || !deprecate_time) {
			return new NV_Error('参数错误');
		}
		const uuid = uuidv4();
		db_invites.insert({
			code: uuid,
			created_time: new Date().getTime(),
			power,deprecate_time,
		})
		return uuid;
	}

	global.nv_get_invite_list = ()=>{
		var now = Date.now();
		return db_invites.find(r=>true).map(invite=>{
			var user_id = invite.user_id;
			if (user_id) { invite.username = get_user_by_id(user_id).name; }
			invite.role = get_user_role(invite.power);
			invite.outdated = now > invite.deprecate_time;
			return invite;
		}).reverse();
	}

	global.nv_get_invite = (id_or_code)=>{
		var invite = db_invites.find(r=>r.id == id_or_code || r.code == id_or_code)[0] || false;
		var user_id = invite.user_id;
		if (user_id) { invite.username = get_user_by_id(user_id).name; }
		invite.role = get_user_role(invite.power);
		invite.outdated = Date.now() > invite.deprecate_time;
		return invite;
	}

	global.nv_delete_invite = (id_or_code)=>{
		db_invites.drop(r=>r.id == id_or_code || r.code == id_or_code);
	}

	global.nv_verify_invite = (id_or_code)=>{
		var invite = nv_get_invite(id_or_code);
		if (!invite) { return new NV_Error('邀请码无效'); }
		if (invite.use_time) { return new NV_Error('邀请码已被使用'); }
		if (new Date().getTime() > invite.deprecate_time) { return new NV_Error('邀请码已过期'); }
		return invite;
	}

	global.nv_revoke_invite = (id_or_code,user_id)=>{
		db_invites.setColumn(r=>r.id == id_or_code || r.code == id_or_code,{
			use_time: new Date().getTime(),
			user_id
		})
	}
}